/* WARNING: file is autogenerated */

#include "gcr/gcr-oids.h"

GQuark
_gcr_oid_name_cn_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.4.3");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_basic_constraints_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.19");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_extended_key_usage_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.37");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_subject_key_identifier_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.14");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_key_usage_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.15");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_subject_alt_name_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.17");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_certificate_policies_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.32");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_authority_key_identifier_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.35");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_authority_info_access_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.6.1.5.5.7.1.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_crl_distribution_points_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("2.5.29.31");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_rsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.1.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_sha1_with_rsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.1.5");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_dsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10040.4.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_sha1_with_dsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10040.4.3");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_ec_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.2.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_sha1_with_ecdsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.4.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_sha224_with_ecdsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.4.3.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_sha256_with_ecdsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.4.3.2");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_sha384_with_ecdsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.4.3.3");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkix1_sha512_with_ecdsa_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.4.3.4");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs7_data_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.7.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs7_signed_data_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.7.2");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs7_encrypted_data_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.7.6");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs9_attribute_friendly_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.9.20");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs9_attribute_local_key_id_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.9.21");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs9_attribute_extension_req_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.9.14");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs12_bag_pkcs8_key_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.12.10.1.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs12_bag_pkcs8_encrypted_key_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.12.10.1.2");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs12_bag_certificate_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.12.10.1.3");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_pkcs12_bag_crl_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.113549.1.12.10.1.4");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_alt_name_xmpp_addr_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.6.1.5.5.7.8.5");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_alt_name_dns_srv_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.6.1.5.5.7.8.7");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_secp192r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.3.1.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect163k1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect163r2_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.15");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_secp224r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.33");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect233k1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.26");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect233r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.27");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_secp256r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.840.10045.3.1.7");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect283k1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.16");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect283r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.17");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_secp384r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.34");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect409k1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.36");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect409r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.37");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_secp521r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.35");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_secp571k1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.38");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_ec_sect571r1_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.3.132.0.39");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_2001_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.2.2.19");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_2012_256_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.1.1.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_2012_512_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.1.1.2");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_test_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.2.2.35.0");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_cryptopro_a_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.2.2.35.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_cryptopro_b_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.2.2.35.2");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_cryptopro_c_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.2.2.35.3");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_cryptopro_xcha_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.2.2.36.0");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_cryptopro_xchb_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.2.2.36.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_gc256a_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.1.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_gc256b_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.1.2");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_gc256c_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.1.3");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_gc256d_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.1.4");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_512_test_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.2.0");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_gc512a_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.2.1");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_gc512b_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.2.2");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

GQuark
_gcr_oid_gostr3410_gc512c_get_quark (void)
{
	static size_t inited = 0;
	static GQuark quark = 0;
	if (g_once_init_enter (&inited)) {
		quark = g_quark_from_static_string ("1.2.643.7.1.2.1.2.3");
		g_once_init_leave (&inited, 1);
	}
	return quark;
}

