
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "gcr-enum-types.h"
#include "gcr.h"
#include "gcr-certificate.h"
#include "gcr-certificate-chain.h"
#include "gcr-certificate-extension.h"
#include "gcr-certificate-extensions.h"
#include "gcr-certificate-extension-list.h"
#include "gcr-certificate-field.h"
#include "gcr-certificate-request.h"
#include "gcr-certificate-section.h"
#include "gcr-fingerprint.h"
#include "gcr-importer.h"
#include "gcr-import-interaction.h"
#include "gcr-library.h"
#include "gcr-mock-prompter.h"
#include "gcr-parser.h"
#include "gcr-pkcs11-certificate.h"
#include "gcr-prompt.h"
#include "gcr-secret-exchange.h"
#include "gcr-secure-memory.h"
#include "gcr-simple-certificate.h"
#include "gcr-ssh-askpass.h"
#include "gcr-subject-public-key-info.h"
#include "gcr-system-prompt.h"
#include "gcr-system-prompter.h"
#include "gcr-trust.h"
#include "gcr-types.h"
#include "gcr-unlock-options.h"
#include "gcr-version.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "gcr-certificate-chain.h" */

GType
gcr_certificate_chain_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCR_CERTIFICATE_CHAIN_UNKNOWN), "GCR_CERTIFICATE_CHAIN_UNKNOWN", "unknown" },
        { C_ENUM (GCR_CERTIFICATE_CHAIN_INCOMPLETE), "GCR_CERTIFICATE_CHAIN_INCOMPLETE", "incomplete" },
        { C_ENUM (GCR_CERTIFICATE_CHAIN_DISTRUSTED), "GCR_CERTIFICATE_CHAIN_DISTRUSTED", "distrusted" },
        { C_ENUM (GCR_CERTIFICATE_CHAIN_SELFSIGNED), "GCR_CERTIFICATE_CHAIN_SELFSIGNED", "selfsigned" },
        { C_ENUM (GCR_CERTIFICATE_CHAIN_PINNED), "GCR_CERTIFICATE_CHAIN_PINNED", "pinned" },
        { C_ENUM (GCR_CERTIFICATE_CHAIN_ANCHORED), "GCR_CERTIFICATE_CHAIN_ANCHORED", "anchored" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GcrCertificateChainStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
gcr_certificate_chain_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (GCR_CERTIFICATE_CHAIN_NONE), "GCR_CERTIFICATE_CHAIN_NONE", "none" },
        { C_FLAGS (GCR_CERTIFICATE_CHAIN_NO_LOOKUPS), "GCR_CERTIFICATE_CHAIN_NO_LOOKUPS", "no-lookups" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("GcrCertificateChainFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gcr-certificate-request.h" */

GType
gcr_certificate_request_format_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCR_CERTIFICATE_REQUEST_PKCS10), "GCR_CERTIFICATE_REQUEST_PKCS10", "pkcs10" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GcrCertificateRequestFormat"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gcr-certificate-section.h" */

GType
gcr_certificate_section_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (GCR_CERTIFICATE_SECTION_NONE), "GCR_CERTIFICATE_SECTION_NONE", "none" },
        { C_FLAGS (GCR_CERTIFICATE_SECTION_IMPORTANT), "GCR_CERTIFICATE_SECTION_IMPORTANT", "important" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("GcrCertificateSectionFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gcr-prompt.h" */

GType
gcr_prompt_reply_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCR_PROMPT_REPLY_CANCEL), "GCR_PROMPT_REPLY_CANCEL", "cancel" },
        { C_ENUM (GCR_PROMPT_REPLY_CONTINUE), "GCR_PROMPT_REPLY_CONTINUE", "continue" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GcrPromptReply"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gcr-system-prompt.h" */

GType
gcr_system_prompt_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCR_SYSTEM_PROMPT_IN_PROGRESS), "GCR_SYSTEM_PROMPT_IN_PROGRESS", "progress" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GcrSystemPromptError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gcr-system-prompter.h" */

GType
gcr_system_prompter_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCR_SYSTEM_PROMPTER_SINGLE), "GCR_SYSTEM_PROMPTER_SINGLE", "single" },
        { C_ENUM (GCR_SYSTEM_PROMPTER_MULTIPLE), "GCR_SYSTEM_PROMPTER_MULTIPLE", "multiple" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GcrSystemPrompterMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "gcr-types.h" */

GType
gcr_data_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCR_ERROR_FAILURE), "GCR_ERROR_FAILURE", "failure" },
        { C_ENUM (GCR_ERROR_UNRECOGNIZED), "GCR_ERROR_UNRECOGNIZED", "unrecognized" },
        { C_ENUM (GCR_ERROR_CANCELLED), "GCR_ERROR_CANCELLED", "cancelled" },
        { C_ENUM (GCR_ERROR_LOCKED), "GCR_ERROR_LOCKED", "locked" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GcrDataError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
gcr_data_format_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCR_FORMAT_ALL), "GCR_FORMAT_ALL", "all" },
        { C_ENUM (GCR_FORMAT_INVALID), "GCR_FORMAT_INVALID", "invalid" },
        { C_ENUM (GCR_FORMAT_DER_PRIVATE_KEY), "GCR_FORMAT_DER_PRIVATE_KEY", "der-private-key" },
        { C_ENUM (GCR_FORMAT_DER_PRIVATE_KEY_RSA), "GCR_FORMAT_DER_PRIVATE_KEY_RSA", "der-private-key-rsa" },
        { C_ENUM (GCR_FORMAT_DER_PRIVATE_KEY_DSA), "GCR_FORMAT_DER_PRIVATE_KEY_DSA", "der-private-key-dsa" },
        { C_ENUM (GCR_FORMAT_DER_PRIVATE_KEY_EC), "GCR_FORMAT_DER_PRIVATE_KEY_EC", "der-private-key-ec" },
        { C_ENUM (GCR_FORMAT_DER_SUBJECT_PUBLIC_KEY), "GCR_FORMAT_DER_SUBJECT_PUBLIC_KEY", "der-subject-public-key" },
        { C_ENUM (GCR_FORMAT_DER_CERTIFICATE_X509), "GCR_FORMAT_DER_CERTIFICATE_X509", "der-certificate-x509" },
        { C_ENUM (GCR_FORMAT_DER_PKCS7), "GCR_FORMAT_DER_PKCS7", "der-pkcs7" },
        { C_ENUM (GCR_FORMAT_DER_PKCS8), "GCR_FORMAT_DER_PKCS8", "der-pkcs8" },
        { C_ENUM (GCR_FORMAT_DER_PKCS8_PLAIN), "GCR_FORMAT_DER_PKCS8_PLAIN", "der-pkcs8-plain" },
        { C_ENUM (GCR_FORMAT_DER_PKCS8_ENCRYPTED), "GCR_FORMAT_DER_PKCS8_ENCRYPTED", "der-pkcs8-encrypted" },
        { C_ENUM (GCR_FORMAT_DER_PKCS10), "GCR_FORMAT_DER_PKCS10", "der-pkcs10" },
        { C_ENUM (GCR_FORMAT_DER_SPKAC), "GCR_FORMAT_DER_SPKAC", "der-spkac" },
        { C_ENUM (GCR_FORMAT_BASE64_SPKAC), "GCR_FORMAT_BASE64_SPKAC", "base64-spkac" },
        { C_ENUM (GCR_FORMAT_DER_PKCS12), "GCR_FORMAT_DER_PKCS12", "der-pkcs12" },
        { C_ENUM (GCR_FORMAT_OPENSSH_PUBLIC), "GCR_FORMAT_OPENSSH_PUBLIC", "openssh-public" },
        { C_ENUM (GCR_FORMAT_OPENPGP_PACKET), "GCR_FORMAT_OPENPGP_PACKET", "openpgp-packet" },
        { C_ENUM (GCR_FORMAT_OPENPGP_ARMOR), "GCR_FORMAT_OPENPGP_ARMOR", "openpgp-armor" },
        { C_ENUM (GCR_FORMAT_PEM), "GCR_FORMAT_PEM", "pem" },
        { C_ENUM (GCR_FORMAT_PEM_PRIVATE_KEY_RSA), "GCR_FORMAT_PEM_PRIVATE_KEY_RSA", "pem-private-key-rsa" },
        { C_ENUM (GCR_FORMAT_PEM_PRIVATE_KEY_DSA), "GCR_FORMAT_PEM_PRIVATE_KEY_DSA", "pem-private-key-dsa" },
        { C_ENUM (GCR_FORMAT_PEM_CERTIFICATE_X509), "GCR_FORMAT_PEM_CERTIFICATE_X509", "pem-certificate-x509" },
        { C_ENUM (GCR_FORMAT_PEM_PKCS7), "GCR_FORMAT_PEM_PKCS7", "pem-pkcs7" },
        { C_ENUM (GCR_FORMAT_PEM_PKCS8_PLAIN), "GCR_FORMAT_PEM_PKCS8_PLAIN", "pem-pkcs8-plain" },
        { C_ENUM (GCR_FORMAT_PEM_PKCS8_ENCRYPTED), "GCR_FORMAT_PEM_PKCS8_ENCRYPTED", "pem-pkcs8-encrypted" },
        { C_ENUM (GCR_FORMAT_PEM_PKCS12), "GCR_FORMAT_PEM_PKCS12", "pem-pkcs12" },
        { C_ENUM (GCR_FORMAT_PEM_PRIVATE_KEY), "GCR_FORMAT_PEM_PRIVATE_KEY", "pem-private-key" },
        { C_ENUM (GCR_FORMAT_PEM_PKCS10), "GCR_FORMAT_PEM_PKCS10", "pem-pkcs10" },
        { C_ENUM (GCR_FORMAT_PEM_PRIVATE_KEY_EC), "GCR_FORMAT_PEM_PRIVATE_KEY_EC", "pem-private-key-ec" },
        { C_ENUM (GCR_FORMAT_PEM_PUBLIC_KEY), "GCR_FORMAT_PEM_PUBLIC_KEY", "pem-public-key" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GcrDataFormat"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

