/* WARNING: file is autogenerated */

#include <glib.h>

#ifndef GCR_OIDS_
#define GCR_OIDS_

GQuark _gcr_oid_name_cn_get_quark (void) G_GNUC_CONST;
#define GCR_OID_NAME_CN _gcr_oid_name_cn_get_quark ()

GQuark _gcr_oid_basic_constraints_get_quark (void) G_GNUC_CONST;
#define GCR_OID_BASIC_CONSTRAINTS _gcr_oid_basic_constraints_get_quark ()

GQuark _gcr_oid_extended_key_usage_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EXTENDED_KEY_USAGE _gcr_oid_extended_key_usage_get_quark ()

GQuark _gcr_oid_subject_key_identifier_get_quark (void) G_GNUC_CONST;
#define GCR_OID_SUBJECT_KEY_IDENTIFIER _gcr_oid_subject_key_identifier_get_quark ()

GQuark _gcr_oid_key_usage_get_quark (void) G_GNUC_CONST;
#define GCR_OID_KEY_USAGE _gcr_oid_key_usage_get_quark ()

GQuark _gcr_oid_subject_alt_name_get_quark (void) G_GNUC_CONST;
#define GCR_OID_SUBJECT_ALT_NAME _gcr_oid_subject_alt_name_get_quark ()

GQuark _gcr_oid_certificate_policies_get_quark (void) G_GNUC_CONST;
#define GCR_OID_CERTIFICATE_POLICIES _gcr_oid_certificate_policies_get_quark ()

GQuark _gcr_oid_authority_key_identifier_get_quark (void) G_GNUC_CONST;
#define GCR_OID_AUTHORITY_KEY_IDENTIFIER _gcr_oid_authority_key_identifier_get_quark ()

GQuark _gcr_oid_authority_info_access_get_quark (void) G_GNUC_CONST;
#define GCR_OID_AUTHORITY_INFO_ACCESS _gcr_oid_authority_info_access_get_quark ()

GQuark _gcr_oid_crl_distribution_points_get_quark (void) G_GNUC_CONST;
#define GCR_OID_CRL_DISTRIBUTION_POINTS _gcr_oid_crl_distribution_points_get_quark ()

GQuark _gcr_oid_pkix1_rsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_RSA _gcr_oid_pkix1_rsa_get_quark ()

GQuark _gcr_oid_pkix1_sha1_with_rsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_SHA1_WITH_RSA _gcr_oid_pkix1_sha1_with_rsa_get_quark ()

GQuark _gcr_oid_pkix1_dsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_DSA _gcr_oid_pkix1_dsa_get_quark ()

GQuark _gcr_oid_pkix1_sha1_with_dsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_SHA1_WITH_DSA _gcr_oid_pkix1_sha1_with_dsa_get_quark ()

GQuark _gcr_oid_pkix1_ec_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_EC _gcr_oid_pkix1_ec_get_quark ()

GQuark _gcr_oid_pkix1_sha1_with_ecdsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_SHA1_WITH_ECDSA _gcr_oid_pkix1_sha1_with_ecdsa_get_quark ()

GQuark _gcr_oid_pkix1_sha224_with_ecdsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_SHA224_WITH_ECDSA _gcr_oid_pkix1_sha224_with_ecdsa_get_quark ()

GQuark _gcr_oid_pkix1_sha256_with_ecdsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_SHA256_WITH_ECDSA _gcr_oid_pkix1_sha256_with_ecdsa_get_quark ()

GQuark _gcr_oid_pkix1_sha384_with_ecdsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_SHA384_WITH_ECDSA _gcr_oid_pkix1_sha384_with_ecdsa_get_quark ()

GQuark _gcr_oid_pkix1_sha512_with_ecdsa_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKIX1_SHA512_WITH_ECDSA _gcr_oid_pkix1_sha512_with_ecdsa_get_quark ()

GQuark _gcr_oid_pkcs7_data_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS7_DATA _gcr_oid_pkcs7_data_get_quark ()

GQuark _gcr_oid_pkcs7_signed_data_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS7_SIGNED_DATA _gcr_oid_pkcs7_signed_data_get_quark ()

GQuark _gcr_oid_pkcs7_encrypted_data_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS7_ENCRYPTED_DATA _gcr_oid_pkcs7_encrypted_data_get_quark ()

GQuark _gcr_oid_pkcs9_attribute_friendly_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS9_ATTRIBUTE_FRIENDLY _gcr_oid_pkcs9_attribute_friendly_get_quark ()

GQuark _gcr_oid_pkcs9_attribute_local_key_id_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS9_ATTRIBUTE_LOCAL_KEY_ID _gcr_oid_pkcs9_attribute_local_key_id_get_quark ()

GQuark _gcr_oid_pkcs9_attribute_extension_req_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS9_ATTRIBUTE_EXTENSION_REQ _gcr_oid_pkcs9_attribute_extension_req_get_quark ()

GQuark _gcr_oid_pkcs12_bag_pkcs8_key_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS12_BAG_PKCS8_KEY _gcr_oid_pkcs12_bag_pkcs8_key_get_quark ()

GQuark _gcr_oid_pkcs12_bag_pkcs8_encrypted_key_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS12_BAG_PKCS8_ENCRYPTED_KEY _gcr_oid_pkcs12_bag_pkcs8_encrypted_key_get_quark ()

GQuark _gcr_oid_pkcs12_bag_certificate_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS12_BAG_CERTIFICATE _gcr_oid_pkcs12_bag_certificate_get_quark ()

GQuark _gcr_oid_pkcs12_bag_crl_get_quark (void) G_GNUC_CONST;
#define GCR_OID_PKCS12_BAG_CRL _gcr_oid_pkcs12_bag_crl_get_quark ()

GQuark _gcr_oid_alt_name_xmpp_addr_get_quark (void) G_GNUC_CONST;
#define GCR_OID_ALT_NAME_XMPP_ADDR _gcr_oid_alt_name_xmpp_addr_get_quark ()

GQuark _gcr_oid_alt_name_dns_srv_get_quark (void) G_GNUC_CONST;
#define GCR_OID_ALT_NAME_DNS_SRV _gcr_oid_alt_name_dns_srv_get_quark ()

GQuark _gcr_oid_ec_secp192r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECP192R1 _gcr_oid_ec_secp192r1_get_quark ()

GQuark _gcr_oid_ec_sect163k1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT163K1 _gcr_oid_ec_sect163k1_get_quark ()

GQuark _gcr_oid_ec_sect163r2_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT163R2 _gcr_oid_ec_sect163r2_get_quark ()

GQuark _gcr_oid_ec_secp224r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECP224R1 _gcr_oid_ec_secp224r1_get_quark ()

GQuark _gcr_oid_ec_sect233k1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT233K1 _gcr_oid_ec_sect233k1_get_quark ()

GQuark _gcr_oid_ec_sect233r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT233R1 _gcr_oid_ec_sect233r1_get_quark ()

GQuark _gcr_oid_ec_secp256r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECP256R1 _gcr_oid_ec_secp256r1_get_quark ()

GQuark _gcr_oid_ec_sect283k1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT283K1 _gcr_oid_ec_sect283k1_get_quark ()

GQuark _gcr_oid_ec_sect283r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT283R1 _gcr_oid_ec_sect283r1_get_quark ()

GQuark _gcr_oid_ec_secp384r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECP384R1 _gcr_oid_ec_secp384r1_get_quark ()

GQuark _gcr_oid_ec_sect409k1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT409K1 _gcr_oid_ec_sect409k1_get_quark ()

GQuark _gcr_oid_ec_sect409r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT409R1 _gcr_oid_ec_sect409r1_get_quark ()

GQuark _gcr_oid_ec_secp521r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECP521R1 _gcr_oid_ec_secp521r1_get_quark ()

GQuark _gcr_oid_ec_secp571k1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECP571K1 _gcr_oid_ec_secp571k1_get_quark ()

GQuark _gcr_oid_ec_sect571r1_get_quark (void) G_GNUC_CONST;
#define GCR_OID_EC_SECT571R1 _gcr_oid_ec_sect571r1_get_quark ()

GQuark _gcr_oid_gostr3410_2001_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_2001 _gcr_oid_gostr3410_2001_get_quark ()

GQuark _gcr_oid_gostr3410_2012_256_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_2012_256 _gcr_oid_gostr3410_2012_256_get_quark ()

GQuark _gcr_oid_gostr3410_2012_512_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_2012_512 _gcr_oid_gostr3410_2012_512_get_quark ()

GQuark _gcr_oid_gostr3410_test_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_TEST _gcr_oid_gostr3410_test_get_quark ()

GQuark _gcr_oid_gostr3410_cryptopro_a_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_CRYPTOPRO_A _gcr_oid_gostr3410_cryptopro_a_get_quark ()

GQuark _gcr_oid_gostr3410_cryptopro_b_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_CRYPTOPRO_B _gcr_oid_gostr3410_cryptopro_b_get_quark ()

GQuark _gcr_oid_gostr3410_cryptopro_c_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_CRYPTOPRO_C _gcr_oid_gostr3410_cryptopro_c_get_quark ()

GQuark _gcr_oid_gostr3410_cryptopro_xcha_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_CRYPTOPRO_XCHA _gcr_oid_gostr3410_cryptopro_xcha_get_quark ()

GQuark _gcr_oid_gostr3410_cryptopro_xchb_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_CRYPTOPRO_XCHB _gcr_oid_gostr3410_cryptopro_xchb_get_quark ()

GQuark _gcr_oid_gostr3410_gc256a_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_GC256A _gcr_oid_gostr3410_gc256a_get_quark ()

GQuark _gcr_oid_gostr3410_gc256b_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_GC256B _gcr_oid_gostr3410_gc256b_get_quark ()

GQuark _gcr_oid_gostr3410_gc256c_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_GC256C _gcr_oid_gostr3410_gc256c_get_quark ()

GQuark _gcr_oid_gostr3410_gc256d_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_GC256D _gcr_oid_gostr3410_gc256d_get_quark ()

GQuark _gcr_oid_gostr3410_512_test_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_512_TEST _gcr_oid_gostr3410_512_test_get_quark ()

GQuark _gcr_oid_gostr3410_gc512a_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_GC512A _gcr_oid_gostr3410_gc512a_get_quark ()

GQuark _gcr_oid_gostr3410_gc512b_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_GC512B _gcr_oid_gostr3410_gc512b_get_quark ()

GQuark _gcr_oid_gostr3410_gc512c_get_quark (void) G_GNUC_CONST;
#define GCR_OID_GOSTR3410_GC512C _gcr_oid_gostr3410_gc512c_get_quark ()


#endif /* GCR_OIDS_ */
