/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __GCR_DBUS_GENERATED_H__
#define __GCR_DBUS_GENERATED_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.keyring.internal.Prompter */

#define _GCR_DBUS_TYPE_PROMPTER (_gcr_dbus_prompter_get_type ())
#define _GCR_DBUS_PROMPTER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), _GCR_DBUS_TYPE_PROMPTER, _GcrDbusPrompter))
#define _GCR_DBUS_IS_PROMPTER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), _GCR_DBUS_TYPE_PROMPTER))
#define _GCR_DBUS_PROMPTER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), _GCR_DBUS_TYPE_PROMPTER, _GcrDbusPrompterIface))

struct __GcrDbusPrompter;
typedef struct __GcrDbusPrompter _GcrDbusPrompter;
typedef struct __GcrDbusPrompterIface _GcrDbusPrompterIface;

struct __GcrDbusPrompterIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_begin_prompting) (
    _GcrDbusPrompter *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_callback);

  gboolean (*handle_perform_prompt) (
    _GcrDbusPrompter *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_callback,
    const gchar *arg_type,
    GVariant *arg_properties,
    const gchar *arg_exchange);

  gboolean (*handle_stop_prompting) (
    _GcrDbusPrompter *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_callback);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (_GcrDbusPrompter, g_object_unref)
#endif

GType _gcr_dbus_prompter_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *_gcr_dbus_prompter_interface_info (void);
guint _gcr_dbus_prompter_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void _gcr_dbus_prompter_complete_begin_prompting (
    _GcrDbusPrompter *object,
    GDBusMethodInvocation *invocation);

void _gcr_dbus_prompter_complete_perform_prompt (
    _GcrDbusPrompter *object,
    GDBusMethodInvocation *invocation);

void _gcr_dbus_prompter_complete_stop_prompting (
    _GcrDbusPrompter *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void _gcr_dbus_prompter_call_begin_prompting (
    _GcrDbusPrompter *proxy,
    const gchar *arg_callback,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean _gcr_dbus_prompter_call_begin_prompting_finish (
    _GcrDbusPrompter *proxy,
    GAsyncResult *res,
    GError **error);

gboolean _gcr_dbus_prompter_call_begin_prompting_sync (
    _GcrDbusPrompter *proxy,
    const gchar *arg_callback,
    GCancellable *cancellable,
    GError **error);

void _gcr_dbus_prompter_call_perform_prompt (
    _GcrDbusPrompter *proxy,
    const gchar *arg_callback,
    const gchar *arg_type,
    GVariant *arg_properties,
    const gchar *arg_exchange,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean _gcr_dbus_prompter_call_perform_prompt_finish (
    _GcrDbusPrompter *proxy,
    GAsyncResult *res,
    GError **error);

gboolean _gcr_dbus_prompter_call_perform_prompt_sync (
    _GcrDbusPrompter *proxy,
    const gchar *arg_callback,
    const gchar *arg_type,
    GVariant *arg_properties,
    const gchar *arg_exchange,
    GCancellable *cancellable,
    GError **error);

void _gcr_dbus_prompter_call_stop_prompting (
    _GcrDbusPrompter *proxy,
    const gchar *arg_callback,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean _gcr_dbus_prompter_call_stop_prompting_finish (
    _GcrDbusPrompter *proxy,
    GAsyncResult *res,
    GError **error);

gboolean _gcr_dbus_prompter_call_stop_prompting_sync (
    _GcrDbusPrompter *proxy,
    const gchar *arg_callback,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define _GCR_DBUS_TYPE_PROMPTER_PROXY (_gcr_dbus_prompter_proxy_get_type ())
#define _GCR_DBUS_PROMPTER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), _GCR_DBUS_TYPE_PROMPTER_PROXY, _GcrDbusPrompterProxy))
#define _GCR_DBUS_PROMPTER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), _GCR_DBUS_TYPE_PROMPTER_PROXY, _GcrDbusPrompterProxyClass))
#define _GCR_DBUS_PROMPTER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), _GCR_DBUS_TYPE_PROMPTER_PROXY, _GcrDbusPrompterProxyClass))
#define _GCR_DBUS_IS_PROMPTER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), _GCR_DBUS_TYPE_PROMPTER_PROXY))
#define _GCR_DBUS_IS_PROMPTER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), _GCR_DBUS_TYPE_PROMPTER_PROXY))

typedef struct __GcrDbusPrompterProxy _GcrDbusPrompterProxy;
typedef struct __GcrDbusPrompterProxyClass _GcrDbusPrompterProxyClass;
typedef struct __GcrDbusPrompterProxyPrivate _GcrDbusPrompterProxyPrivate;

struct __GcrDbusPrompterProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  _GcrDbusPrompterProxyPrivate *priv;
};

struct __GcrDbusPrompterProxyClass
{
  GDBusProxyClass parent_class;
};

GType _gcr_dbus_prompter_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (_GcrDbusPrompterProxy, g_object_unref)
#endif

void _gcr_dbus_prompter_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
_GcrDbusPrompter *_gcr_dbus_prompter_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
_GcrDbusPrompter *_gcr_dbus_prompter_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void _gcr_dbus_prompter_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
_GcrDbusPrompter *_gcr_dbus_prompter_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
_GcrDbusPrompter *_gcr_dbus_prompter_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define _GCR_DBUS_TYPE_PROMPTER_SKELETON (_gcr_dbus_prompter_skeleton_get_type ())
#define _GCR_DBUS_PROMPTER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), _GCR_DBUS_TYPE_PROMPTER_SKELETON, _GcrDbusPrompterSkeleton))
#define _GCR_DBUS_PROMPTER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), _GCR_DBUS_TYPE_PROMPTER_SKELETON, _GcrDbusPrompterSkeletonClass))
#define _GCR_DBUS_PROMPTER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), _GCR_DBUS_TYPE_PROMPTER_SKELETON, _GcrDbusPrompterSkeletonClass))
#define _GCR_DBUS_IS_PROMPTER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), _GCR_DBUS_TYPE_PROMPTER_SKELETON))
#define _GCR_DBUS_IS_PROMPTER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), _GCR_DBUS_TYPE_PROMPTER_SKELETON))

typedef struct __GcrDbusPrompterSkeleton _GcrDbusPrompterSkeleton;
typedef struct __GcrDbusPrompterSkeletonClass _GcrDbusPrompterSkeletonClass;
typedef struct __GcrDbusPrompterSkeletonPrivate _GcrDbusPrompterSkeletonPrivate;

struct __GcrDbusPrompterSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  _GcrDbusPrompterSkeletonPrivate *priv;
};

struct __GcrDbusPrompterSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType _gcr_dbus_prompter_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (_GcrDbusPrompterSkeleton, g_object_unref)
#endif

_GcrDbusPrompter *_gcr_dbus_prompter_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.keyring.internal.Prompter.Callback */

#define _GCR_DBUS_TYPE_PROMPTER_CALLBACK (_gcr_dbus_prompter_callback_get_type ())
#define _GCR_DBUS_PROMPTER_CALLBACK(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK, _GcrDbusPrompterCallback))
#define _GCR_DBUS_IS_PROMPTER_CALLBACK(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK))
#define _GCR_DBUS_PROMPTER_CALLBACK_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK, _GcrDbusPrompterCallbackIface))

struct __GcrDbusPrompterCallback;
typedef struct __GcrDbusPrompterCallback _GcrDbusPrompterCallback;
typedef struct __GcrDbusPrompterCallbackIface _GcrDbusPrompterCallbackIface;

struct __GcrDbusPrompterCallbackIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_prompt_done) (
    _GcrDbusPrompterCallback *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_prompt_ready) (
    _GcrDbusPrompterCallback *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_reply,
    GVariant *arg_properties,
    const gchar *arg_exchange);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (_GcrDbusPrompterCallback, g_object_unref)
#endif

GType _gcr_dbus_prompter_callback_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *_gcr_dbus_prompter_callback_interface_info (void);
guint _gcr_dbus_prompter_callback_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void _gcr_dbus_prompter_callback_complete_prompt_ready (
    _GcrDbusPrompterCallback *object,
    GDBusMethodInvocation *invocation);

void _gcr_dbus_prompter_callback_complete_prompt_done (
    _GcrDbusPrompterCallback *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void _gcr_dbus_prompter_callback_call_prompt_ready (
    _GcrDbusPrompterCallback *proxy,
    const gchar *arg_reply,
    GVariant *arg_properties,
    const gchar *arg_exchange,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean _gcr_dbus_prompter_callback_call_prompt_ready_finish (
    _GcrDbusPrompterCallback *proxy,
    GAsyncResult *res,
    GError **error);

gboolean _gcr_dbus_prompter_callback_call_prompt_ready_sync (
    _GcrDbusPrompterCallback *proxy,
    const gchar *arg_reply,
    GVariant *arg_properties,
    const gchar *arg_exchange,
    GCancellable *cancellable,
    GError **error);

void _gcr_dbus_prompter_callback_call_prompt_done (
    _GcrDbusPrompterCallback *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean _gcr_dbus_prompter_callback_call_prompt_done_finish (
    _GcrDbusPrompterCallback *proxy,
    GAsyncResult *res,
    GError **error);

gboolean _gcr_dbus_prompter_callback_call_prompt_done_sync (
    _GcrDbusPrompterCallback *proxy,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define _GCR_DBUS_TYPE_PROMPTER_CALLBACK_PROXY (_gcr_dbus_prompter_callback_proxy_get_type ())
#define _GCR_DBUS_PROMPTER_CALLBACK_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_PROXY, _GcrDbusPrompterCallbackProxy))
#define _GCR_DBUS_PROMPTER_CALLBACK_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_PROXY, _GcrDbusPrompterCallbackProxyClass))
#define _GCR_DBUS_PROMPTER_CALLBACK_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_PROXY, _GcrDbusPrompterCallbackProxyClass))
#define _GCR_DBUS_IS_PROMPTER_CALLBACK_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_PROXY))
#define _GCR_DBUS_IS_PROMPTER_CALLBACK_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_PROXY))

typedef struct __GcrDbusPrompterCallbackProxy _GcrDbusPrompterCallbackProxy;
typedef struct __GcrDbusPrompterCallbackProxyClass _GcrDbusPrompterCallbackProxyClass;
typedef struct __GcrDbusPrompterCallbackProxyPrivate _GcrDbusPrompterCallbackProxyPrivate;

struct __GcrDbusPrompterCallbackProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  _GcrDbusPrompterCallbackProxyPrivate *priv;
};

struct __GcrDbusPrompterCallbackProxyClass
{
  GDBusProxyClass parent_class;
};

GType _gcr_dbus_prompter_callback_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (_GcrDbusPrompterCallbackProxy, g_object_unref)
#endif

void _gcr_dbus_prompter_callback_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
_GcrDbusPrompterCallback *_gcr_dbus_prompter_callback_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
_GcrDbusPrompterCallback *_gcr_dbus_prompter_callback_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void _gcr_dbus_prompter_callback_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
_GcrDbusPrompterCallback *_gcr_dbus_prompter_callback_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
_GcrDbusPrompterCallback *_gcr_dbus_prompter_callback_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define _GCR_DBUS_TYPE_PROMPTER_CALLBACK_SKELETON (_gcr_dbus_prompter_callback_skeleton_get_type ())
#define _GCR_DBUS_PROMPTER_CALLBACK_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_SKELETON, _GcrDbusPrompterCallbackSkeleton))
#define _GCR_DBUS_PROMPTER_CALLBACK_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_SKELETON, _GcrDbusPrompterCallbackSkeletonClass))
#define _GCR_DBUS_PROMPTER_CALLBACK_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_SKELETON, _GcrDbusPrompterCallbackSkeletonClass))
#define _GCR_DBUS_IS_PROMPTER_CALLBACK_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_SKELETON))
#define _GCR_DBUS_IS_PROMPTER_CALLBACK_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), _GCR_DBUS_TYPE_PROMPTER_CALLBACK_SKELETON))

typedef struct __GcrDbusPrompterCallbackSkeleton _GcrDbusPrompterCallbackSkeleton;
typedef struct __GcrDbusPrompterCallbackSkeletonClass _GcrDbusPrompterCallbackSkeletonClass;
typedef struct __GcrDbusPrompterCallbackSkeletonPrivate _GcrDbusPrompterCallbackSkeletonPrivate;

struct __GcrDbusPrompterCallbackSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  _GcrDbusPrompterCallbackSkeletonPrivate *priv;
};

struct __GcrDbusPrompterCallbackSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType _gcr_dbus_prompter_callback_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (_GcrDbusPrompterCallbackSkeleton, g_object_unref)
#endif

_GcrDbusPrompterCallback *_gcr_dbus_prompter_callback_skeleton_new (void);


G_END_DECLS

#endif /* __GCR_DBUS_GENERATED_H__ */
