/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* BOOLEAN:STRING,POINTER (../gcr/gck/gck-marshal.list:1) */
extern
void _gck_marshal_BOOLEAN__STRING_POINTER (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
                                           const GValue *param_values,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);

/* BOOLEAN:OBJECT,STRING,POINTER (../gcr/gck/gck-marshal.list:2) */
extern
void _gck_marshal_BOOLEAN__OBJECT_STRING_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
                                                  const GValue *param_values,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);

/* BOOLEAN:ULONG (../gcr/gck/gck-marshal.list:3) */
extern
void _gck_marshal_BOOLEAN__ULONG (GClosure     *closure,
                                  GValue       *return_value,
                                  guint         n_param_values,
                                  const GValue *param_values,
                                  gpointer      invocation_hint,
                                  gpointer      marshal_data);


G_END_DECLS
