
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "gck-enum-types.h"
#include "gck.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "gck.h" */

GType
gck_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCK_ERROR_MODULE_PROBLEM), "GCK_ERROR_MODULE_PROBLEM", "problem" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GckError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
gck_session_options_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (GCK_SESSION_READ_ONLY), "GCK_SESSION_READ_ONLY", "read-only" },
        { C_FLAGS (GCK_SESSION_READ_WRITE), "GCK_SESSION_READ_WRITE", "read-write" },
        { C_FLAGS (GCK_SESSION_LOGIN_USER), "GCK_SESSION_LOGIN_USER", "login-user" },
        { C_FLAGS (GCK_SESSION_AUTHENTICATE), "GCK_SESSION_AUTHENTICATE", "authenticate" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("GckSessionOptions"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
gck_builder_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCK_BUILDER_NONE), "GCK_BUILDER_NONE", "none" },
        { C_ENUM (GCK_BUILDER_SECURE_MEMORY), "GCK_BUILDER_SECURE_MEMORY", "secure-memory" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GckBuilderFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
gck_uri_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (GCK_URI_BAD_SCHEME), "GCK_URI_BAD_SCHEME", "bad-scheme" },
        { C_ENUM (GCK_URI_BAD_ENCODING), "GCK_URI_BAD_ENCODING", "bad-encoding" },
        { C_ENUM (GCK_URI_BAD_SYNTAX), "GCK_URI_BAD_SYNTAX", "bad-syntax" },
        { C_ENUM (GCK_URI_BAD_VERSION), "GCK_URI_BAD_VERSION", "bad-version" },
        { C_ENUM (GCK_URI_NOT_FOUND), "GCK_URI_NOT_FOUND", "not-found" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GckUriError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
gck_uri_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (GCK_URI_FOR_OBJECT), "GCK_URI_FOR_OBJECT", "for-object" },
        { C_FLAGS (GCK_URI_FOR_TOKEN), "GCK_URI_FOR_TOKEN", "for-token" },
        { C_FLAGS (GCK_URI_FOR_MODULE), "GCK_URI_FOR_MODULE", "for-module" },
        { C_FLAGS (GCK_URI_WITH_VERSION), "GCK_URI_WITH_VERSION", "with-version" },
        { C_FLAGS (GCK_URI_FOR_ANY), "GCK_URI_FOR_ANY", "for-any" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("GckUriFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

